/*
  Copyright (c) 1998 - 2008
  ILK  -  Tilburg University
  CNTS -  University of Antwerp
 
  This file is part of Timbl

  Timbl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.

  Timbl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.

  For questions and suggestions, see:
      http://ilk.uvt.nl/software.html
  or send mail to:
      Timbl@uvt.nl
*/

#include <cassert>
#include <string>
#include <vector>
#include <ostream>
#include <iostream>

#include "timbl/CommandLine.h"

using namespace std;

namespace Timbl {

  CL_Options::CL_Options( const int argc, const char * const *argv ):
    Opts( Split_Command_Line( argc, argv ) ){
    if ( !Opts )
      exit(99);
  }
  
  CL_Options::CL_Options( const string& args ){
    const char *argstr = args.c_str();
    Opts = Split_Command_Line( 0, &argstr );
    if ( !Opts )
      exit(99);
  }
  
  CL_Options::~CL_Options(){
    delete Opts;
  }
  
  ostream& operator<<( ostream& os, const CL_Options& cl ){
    list<CL_item>::iterator pos;
    for ( pos = cl.Opts->begin(); pos != cl.Opts->end(); ++pos ){
      os << *pos << " ";
    }
    return os;
  }

  bool CL_Options::Present( const char c ) const {
    list<CL_item>::const_iterator pos;
    for ( pos = Opts->begin(); pos != Opts->end(); ++pos ){
      if ( pos->OptChar() == c ){
	return true;
      }
    }
    return false;
  }
  
  bool CL_Options::Find( const char c, string &opt, bool& mood ) const {
    list<CL_item>::const_iterator pos;
    for ( pos = Opts->begin(); pos != Opts->end(); ++pos ){
      if ( pos->OptChar() == c ){
	opt = pos->Option();
	mood = pos->Mood();
	return true;
      }
    }
    return false;
  }
  
  bool CL_Options::Delete( const char c, bool all ){
    bool result = Present( c );
    if ( result ) {
      do {
	list<CL_item>::iterator pos;
	for ( pos = Opts->begin(); pos != Opts->end(); ++pos ){
	  if ( pos->OptChar() == c ){
	    Opts->erase(pos);
	    break; // for
	  }
	}
      } while ( all && Present(c) );
    }
    return result;
  }
  
  void CL_Options::Add( const string& s, const string& line, bool mood ){
    CL_item cl( s, line, mood );
    Opts->push_front( cl );
  }

  void CL_Options::Add( const char c, const string& line, bool mood ){
    CL_item cl( c, line, mood );
    Opts->push_front( cl );
  }

  inline bool p_or_m( char k )
  { return ( k == '+' || k == '-' ); }

  inline int opt_split( const char *line, vector<string>& new_argv ){ 
    int k=0;
    const char *p = line;
    int argc = 0;
    while ( *p ){
      if ( ( p_or_m(*p) && argc == 0 ) || 
	   ( isspace(*p++) && p_or_m(*p) ) ){
	argc++;
      }
    }
    string res;
    if ( argc != 0 ){
      new_argv.reserve(argc);
      p = line;
      while ( isspace( *p ) ){ p++; };
      while ( *p ){
	int skip = 0;
	while ( isspace( *p ) ){ p++; skip++; };
	if ( !*p )
	  break;
	if ( skip != 0 && p_or_m(*p) && k != 1 ){
	  new_argv.push_back( res );
	  k = 0;
	  res = "";
	}
	res += *p++;
      }
      new_argv.push_back( res );
    }
    return argc;
  }
  
  CommandLine *Split_Command_Line( const int Argc, 
				   const char * const *Argv ){
    int local_argc = 0;
    vector<string> local_argv;
    char Optchar;
    string Optword;
    string Option;
    bool Mood = false;
    if ( Argc == 0 ) 
      if ( Argv != NULL &&
	   Argv[0] != NULL ){
	local_argc = opt_split( Argv[0], local_argv );
      }
      else
	return NULL;
    else {
      local_argc = Argc-1;
      for( int i=1; i < Argc; ++i ){
	// start at 1 to skip the program name
	local_argv.push_back( Argv[i] );
      }
    }
    CommandLine *result = new CommandLine();
    for ( int arg_ind=0; arg_ind < local_argc; ++arg_ind ){
      Option = local_argv[arg_ind];
      if ( !p_or_m(Option[0]) ){
	Optchar = '?';
	Optword = Option;
	Mood = false;
      }
      else {
	Mood = Option[0] == '+';
	if ( Option.size() > 1 ){
	  Optchar = Option[1];
	  Optword = Option;
	  Optword = Optword.erase(0,1);
	  string::size_type pos = Optword.find( "=" );
	  if ( pos == string::npos )
	    Option = Option.erase(0,2);
	  else {
	    Option = Optword.substr( pos+1 );
	    Optword = Optword.substr( 0, pos );
	  }
	}
	else {
	  Optchar = 0;
	  Optword = Option;
	  Option = Option.erase(0,1);
	}
	if ( (!Optchar || Option.empty() ) && arg_ind+1 < local_argc ) {
	  string tmpOption = local_argv[arg_ind+1];
	  if ( !p_or_m(tmpOption[0]) ){
	    Option = tmpOption;
	    ++arg_ind;
	    if ( !Optchar ){
	      Optchar = Option[1];
	      Option = Option.erase(0,2);
	    }
	  }
	}
      }
      CL_item cl( Optword, Option, Mood );
      result->push_front( cl );
    }
    return result;
  }
    
}


